import torch
import torch.nn as nn

class B_RNN(nn.Module):
    def __init__(self, num_classes, input_size, hidden_size, num_layers=1):
        '''

        :param num_classes: label有多少个类别
        :param input_size: C, 将被H替代
        :param hidden_size: H, 用于取代C
        :param num_layers: 多少层RNN
        '''
        super().__init__()
        self.rnn = nn.RNN(
            input_size=input_size,
            hidden_size=hidden_size,
            num_layers=num_layers,
            batch_first=True
        )
        self.fc = nn.Linear(hidden_size, num_classes)

    def forward(self, x):  # x [B, C, L]
        x = x.permute(0, 2, 1)  # x [B, L, C]

        x, (h, _) = self.lstm(x)
        # x [B, L, H]
        # h [num_layers * num_directions, B, H]

        # 法一: 取最后一个时间步的输出作为整个序列的表示
        out = x[:, -1, :]  # out [B, H]

        # 法二: 取最后一层的 hidden state 作为整个序列的表示
        # out = h[-1]  # out [B, H]

        out = self.fc(out)  # out [B, num_classes]

        return out

if __name__ == '__main__':
    model = B_RNN(num_classes=11, input_size=2, hidden_size=512, num_layers=4)
    x = torch.randn(64, 2, 128) # 输入形状 [batch_size, input_size, seq_len]
    y = model(x)
    print(y.shape)